@@ -154,12 +154,14 @@ class NoneStream(RawIOBase):
         return None
 
 
-default_bytes_out_stream = getattr(
-        sys.stdout, 'buffer', sys.stdout
-    ) or NoneStream()
-default_bytes_err_stream = getattr(
-        sys.stderr, 'buffer', sys.stderr
-    ) or NoneStream()
+if sys.version_info[0] == 2:
+    default_bytes_out_stream = sys.stdout or NoneStream()
+    default_bytes_err_stream = sys.stderr or NoneStream()
+else:
+    default_bytes_out_stream = (
+        getattr(sys.stdout, 'buffer', None) or NoneStream())
+    default_bytes_err_stream = (
+        getattr(sys.stderr, 'buffer', None) or NoneStream())
 
 
 DEFAULT_ENCODING = 'utf-8'
@@ -406,6 +408,57 @@ def add(repo=".", paths=None):
     return (relpaths, ignored)
 
 
+def _is_subdir(subdir, parentdir):
+    """Check whether subdir is parentdir or a subdir of parentdir
+
+        If parentdir or subdir is a relative path, it will be disamgibuated
+        relative to the pwd.
+    """
+    parentdir_abs = os.path.realpath(parentdir) + os.path.sep
+    subdir_abs = os.path.realpath(subdir) + os.path.sep
+    return subdir_abs.startswith(parentdir_abs)
+
+
+# TODO: option to remove ignored files also, in line with `git clean -fdx`
+def clean(repo=".", target_dir=None):
+    """Remove any untracked files from the target directory recursively
+
+    Equivalent to running `git clean -fd` in target_dir.
+
+    :param repo: Repository where the files may be tracked
+    :param target_dir: Directory to clean - current directory if None
+    """
+    if target_dir is None:
+        target_dir = os.getcwd()
+
+    with open_repo_closing(repo) as r:
+        if not _is_subdir(target_dir, r.path):
+            raise ValueError("target_dir must be in the repo's working dir")
+
+        index = r.open_index()
+        ignore_manager = IgnoreFilterManager.from_repo(r)
+
+        paths_in_wd = _walk_working_dir_paths(target_dir, r.path)
+        # Reverse file visit order, so that files and subdirectories are
+        # removed before containing directory
+        for ap, is_dir in reversed(list(paths_in_wd)):
+            if is_dir:
+                # All subdirectories and files have been removed if untracked,
+                # so dir contains no tracked files iff it is empty.
+                is_empty = len(os.listdir(ap)) == 0
+                if is_empty:
+                    os.rmdir(ap)
+            else:
+                ip = path_to_tree_path(r.path, ap)
+                is_tracked = ip in index
+
+                rp = os.path.relpath(ap, r.path)
+                is_ignored = ignore_manager.is_ignored(rp)
+
+                if not is_tracked and not is_ignored:
+                    os.remove(ap)
+
+
 def remove(repo=".", paths=None, cached=False):
     """Remove files from the staging area.
 
@@ -494,7 +547,10 @@ def print_tag(tag, decode, outstream=sys.stdout):
     :param outstream: A stream to write to
     """
     outstream.write("Tagger: " + decode(tag.tagger) + "\n")
-    outstream.write("Date:   " + decode(tag.tag_time) + "\n")
+    time_tuple = time.gmtime(tag.tag_time + tag.tag_timezone)
+    time_str = time.strftime("%a %b %d %Y %H:%M:%S", time_tuple)
+    timezone_str = format_timezone(tag.tag_timezone).decode('ascii')
+    outstream.write("Date:   " + time_str + " " + timezone_str + "\n")
     outstream.write("\n")
     outstream.write(decode(tag.message) + "\n")
     outstream.write("\n")
@@ -556,7 +612,7 @@ def show_tag(repo, tag, decode, outstream=sys.stdout):
     :param outstream: Stream to write to
     """
     print_tag(tag, decode, outstream)
-    show_object(repo, repo[tag.object[1]], outstream)
+    show_object(repo, repo[tag.object[1]], decode, outstream)
 
 
 def show_object(repo, obj, decode, outstream):
@@ -721,7 +777,8 @@ def tag_create(
             if sign:
                 import gpg
                 with gpg.Context(armor=True) as c:
-                    tag_obj.signature, result = c.sign(tag_obj.as_raw_string())
+                    tag_obj.signature, unused_result = c.sign(
+                        tag_obj.as_raw_string())
             r.object_store.add_object(tag_obj)
             tag_id = tag_obj.id
         else:
@@ -873,7 +930,7 @@ def status(repo=".", ignored=False):
     :param repo: Path to repository or repository object
     :param ignored: Whether to include ignored files in `untracked`
     :return: GitStatus tuple,
-        staged -    list of staged paths (diff index/HEAD)
+        staged -  dict with lists of staged paths (diff index/HEAD)
         unstaged -  list of unstaged paths (diff index/working-tree)
         untracked - list of untracked, un-ignored & non-.git paths
     """
@@ -898,14 +955,12 @@ def status(repo=".", ignored=False):
         return GitStatus(tracked_changes, unstaged_changes, untracked_changes)
 
 
-def get_untracked_paths(frompath, basepath, index):
-    """Get untracked paths.
+def _walk_working_dir_paths(frompath, basepath):
+    """Get path, is_dir for files in working dir from frompath
 
-    ;param frompath: Path to walk
+    :param frompath: Path to begin walk
     :param basepath: Path to compare to
-    :param index: Index to check against
     """
-    # If nothing is specified, add all non-ignored files.
     for dirpath, dirnames, filenames in os.walk(frompath):
         # Skip .git and below.
         if '.git' in dirnames:
@@ -916,8 +971,24 @@ def get_untracked_paths(frompath, basepath, index):
             filenames.remove('.git')
             if dirpath != basepath:
                 continue
+
+        if dirpath != frompath:
+            yield dirpath, True
+
         for filename in filenames:
-            ap = os.path.join(dirpath, filename)
+            filepath = os.path.join(dirpath, filename)
+            yield filepath, False
+
+
+def get_untracked_paths(frompath, basepath, index):
+    """Get untracked paths.
+
+    ;param frompath: Path to walk
+    :param basepath: Path to compare to
+    :param index: Index to check against
+    """
+    for ap, is_dir in _walk_working_dir_paths(frompath, basepath):
+        if not is_dir:
             ip = path_to_tree_path(basepath, ap)
             if ip not in index:
                 yield os.path.relpath(ap, frompath)
